#ifndef DUNE_TECTONIC_GLOBALRUINANONLINEARITY_HH
#define DUNE_TECTONIC_GLOBALRUINANONLINEARITY_HH

#include <vector>

#include <dune/common/bitsetvector.hh>
#include <dune/common/fmatrix.hh>
#include <dune/common/fvector.hh>
#include <dune/grid/common/mcmgmapper.hh>
#include <dune/istl/bcrsmatrix.hh>
#include <dune/istl/bvector.hh>

#include <dune/tectonic/globalfrictiondata.hh>

#include "globalnonlinearity.hh"
#include "frictionpotential.hh"

template <class Matrix, class Vector, class GridView>
class GlobalRuinaNonlinearity : public GlobalNonlinearity<Matrix, Vector> {
public:
  using GlobalNonlinearity<Matrix, Vector>::block_size;
  using typename GlobalNonlinearity<Matrix, Vector>::Friction;

private:
  using typename GlobalNonlinearity<Matrix, Vector>::ScalarVector;

public:
  GlobalRuinaNonlinearity(BoundaryPatch<GridView> const &frictionalBoundary,
                          GridView const &gridView,
                          GlobalFrictionData<block_size> const &frictionInfo,
                          // Note: passing the following two makes no sense
                          ScalarVector const &weights,
                          ScalarVector const &normalStress)
      : restrictions(normalStress.size()) {
    auto trivialNonlinearity =
        std::make_shared<Friction>(std::make_shared<TrivialFunction>());

    Dune::MultipleCodimMultipleGeomTypeMapper<
        GridView, Dune::MCMGVertexLayout> const vertexMapper(gridView);
    for (auto it = gridView.template begin<block_size>();
         it != gridView.template end<block_size>(); ++it) {
      auto const i = vertexMapper.map(*it);
      auto const coordinate = it->geometry().corner(0);
      if (not frictionalBoundary.containsVertex(i)) {
        restrictions[i] = trivialNonlinearity;
        continue;
      }
      auto const fp = std::make_shared<FrictionPotential>(
          weights[i], normalStress[i], frictionInfo(coordinate));
      restrictions[i] = std::make_shared<Friction>(fp);
    }
  }

  void updateLogState(ScalarVector const &logState) override {
    for (size_t i = 0; i < restrictions.size(); ++i)
      restrictions[i]->updateLogState(logState[i]);
  }

  void coefficientOfFriction(Vector const &x, ScalarVector &coefficient) {
    assert(x.size() == restrictions.size());
    coefficient.resize(restrictions.size());
    for (size_t i = 0; i < restrictions.size(); ++i)
      coefficient[i] = restrictions[i]->coefficientOfFriction(x[i]);
  }

  /*
    Return a restriction of the outer function to the i'th node.
  */
  std::shared_ptr<Friction> restriction(size_t i) const override {
    return restrictions[i];
  }

private:
  std::vector<std::shared_ptr<Friction>> restrictions;
};
#endif
